@@ -10,11 +10,17 @@ TestApp->config( name => 'TestApp', root => '/some/dir' );
 
 TestApp->setup;
 
+sub index : Private {
+    my ( $self, $c ) = @_;
+    $c->res->body( 'root index' );
+}
+
 sub global_action : Private {
     my ( $self, $c ) = @_;
     $c->forward('TestApp::View::Dump::Request');
 }
 
+
 sub execute {
     my $c      = shift;
     my $class  = ref( $c->component( $_[0] ) ) || $_[0];
@@ -28,8 +34,12 @@ sub execute {
     elsif ( $action =~ /\/(\w+)$/ ) {
         $method = $1;
     }
+    elsif ( $action =~ /^(\w+)$/ ) {
+        $method = $action;
+    }
 
-    if ( $class && $method ) {
+
+    if ( $class && $method && $method !~ /^_/ ) {
         my $executed = sprintf( "%s->%s", $class, $method );
         my @executed = $c->response->headers->header('X-Catalyst-Executed');
         push @executed, $executed;
