@@ -3,12 +3,12 @@ package TestApp::Controller::Action::Regexp;
 use strict;
 use base 'TestApp::Controller::Action';
 
-sub one : Action Regexp('^action/regexp/(\w+)/(\d+)$') {
+sub one : Action Regex('^action/regexp/(\w+)/(\d+)$') {
     my ( $self, $c ) = @_;
     $c->forward('TestApp::View::Dump::Request');
 }
 
-sub two : Action Regex('^action/regexp/(\d+)/(\w+)$') {
+sub two : Action Regexp('(\d+)/(\w+)$') {
     my ( $self, $c ) = @_;
     $c->forward('TestApp::View::Dump::Request');
 }
